@@ -25,6 +25,7 @@
 #include "kateviewinternal.h"
 #include "kateconfig.h"
 #include "katecompletionwidget.h"
+#include <katecompletiontree.h>
 #include "kateglobal.h"
 #include "katevikeyparser.h"
 
@@ -39,6 +40,8 @@ KateViInsertMode::KateViInsertMode( KateViInputModeManager *viInputModeManager,
 
   m_blockInsert = None;
   m_eolPos = 0;
+  m_count = 1;
+  m_countedRepeatsBeginOnNewLine = false;
 }
 
 KateViInsertMode::~KateViInsertMode()
@@ -189,12 +192,15 @@ bool KateViInsertMode::commandMoveOneWordRight()
 
 bool KateViInsertMode::commandCompleteNext()
 {
-  // Completion may yield different results when performed
-  // other places in the document. Therefore, repeat the
-  // inserted text instead of the typed keystrokes.
-  m_viInputModeManager->setTextualRepeat();
   if(m_view->completionWidget()->isCompletionActive()) {
+    const QModelIndex oldCompletionItem = m_view->completionWidget()->treeView()->selectionModel()->currentIndex();
     m_view->completionWidget()->cursorDown();
+    const QModelIndex newCompletionItem = m_view->completionWidget()->treeView()->selectionModel()->currentIndex();
+    if (newCompletionItem == oldCompletionItem)
+    {
+      // Wrap to top.
+      m_view->completionWidget()->top();
+    }
   } else {
     m_view->userInvokedCompletion();
   }
@@ -203,9 +209,15 @@ bool KateViInsertMode::commandCompleteNext()
 
 bool KateViInsertMode::commandCompletePrevious()
 {
-  m_viInputModeManager->setTextualRepeat();
   if(m_view->completionWidget()->isCompletionActive()) {
+    const QModelIndex oldCompletionItem = m_view->completionWidget()->treeView()->selectionModel()->currentIndex();
     m_view->completionWidget()->cursorUp();
+    const QModelIndex newCompletionItem = m_view->completionWidget()->treeView()->selectionModel()->currentIndex();
+    if (newCompletionItem == oldCompletionItem)
+    {
+      // Wrap to bottom.
+      m_view->completionWidget()->bottom();
+    }
   } else {
     m_view->userInvokedCompletion();
     m_view->completionWidget()->bottom();
@@ -234,6 +246,10 @@ bool KateViInsertMode::commandInsertContentOfRegister(){
       cAfter.setLine( cAfter.line()+1 );
       cAfter.setColumn( 0 );
     }
+    else
+    {
+      cAfter.setColumn(cAfter.column() + textToInsert.length());
+    }
 
     doc()->insertText( c, textToInsert, m == Block );
 
@@ -246,6 +262,12 @@ bool KateViInsertMode::commandInsertContentOfRegister(){
 bool KateViInsertMode::commandSwitchToNormalModeForJustOneCommand(){
     m_viInputModeManager->setTemporaryNormalMode(true);
     m_viInputModeManager->changeViMode(NormalMode);
+    const Cursor cursorPos = m_view->cursorPosition();
+    // If we're at end of the line, move the cursor back one step, as in Vim.
+    if (doc()->line(cursorPos.line()).length() == cursorPos.column())
+    {
+      m_view->setCursorPosition(Cursor(cursorPos.line(), cursorPos.column() - 1));
+    }
     m_view->setCaretStyle( KateRenderer::Block, true );
     m_view->updateViModeBarMode();
     m_viewInternal->repaint();
@@ -416,50 +438,70 @@ bool KateViInsertMode::handleKeypress( const QKeyEvent *e )
 // ctrl-c is used to exit insert mode this is not done.
 void KateViInsertMode::leaveInsertMode( bool force )
 {
-    if ( !force && m_blockInsert != None ) { // block append/prepend
+    if ( !force )
+    {
+      if ( m_blockInsert != None ) { // block append/prepend
 
-        // make sure cursor haven't been moved
-        if ( m_blockRange.startLine == m_view->cursorPosition().line() ) {
-            int start, len;
-            QString added;
-            Cursor c;
+          // make sure cursor haven't been moved
+          if ( m_blockRange.startLine == m_view->cursorPosition().line() ) {
+              int start, len;
+              QString added;
+              Cursor c;
 
-            switch ( m_blockInsert ) {
-            case Append:
-            case Prepend:
-                if ( m_blockInsert == Append ) {
-                    start = m_blockRange.endColumn+1;
-                } else {
-                    start = m_blockRange.startColumn;
-                }
+              switch ( m_blockInsert ) {
+              case Append:
+              case Prepend:
+                  if ( m_blockInsert == Append ) {
+                      start = m_blockRange.endColumn+1;
+                  } else {
+                      start = m_blockRange.startColumn;
+                  }
 
-                len = m_view->cursorPosition().column()-start;
-                added = getLine().mid( start, len );
+                  len = m_view->cursorPosition().column()-start;
+                  added = getLine().mid( start, len );
 
-                c = Cursor( m_blockRange.startLine, start );
-                for ( int i = m_blockRange.startLine+1; i <= m_blockRange.endLine; i++ ) {
-                    c.setLine( i );
-                    doc()->insertText( c, added );
-                }
-                break;
-            case AppendEOL:
-                start = m_eolPos;
-                len = m_view->cursorPosition().column()-start;
-                added = getLine().mid( start, len );
+                  c = Cursor( m_blockRange.startLine, start );
+                  for ( int i = m_blockRange.startLine+1; i <= m_blockRange.endLine; i++ ) {
+                      c.setLine( i );
+                      doc()->insertText( c, added );
+                  }
+                  break;
+              case AppendEOL:
+                  start = m_eolPos;
+                  len = m_view->cursorPosition().column()-start;
+                  added = getLine().mid( start, len );
 
-                c = Cursor( m_blockRange.startLine, start );
-                for ( int i = m_blockRange.startLine+1; i <= m_blockRange.endLine; i++ ) {
-                    c.setLine( i );
-                    c.setColumn( doc()->lineLength( i ) );
-                    doc()->insertText( c, added );
-                }
-                break;
-            default:
-                error("not supported");
-            }
-        }
+                  c = Cursor( m_blockRange.startLine, start );
+                  for ( int i = m_blockRange.startLine+1; i <= m_blockRange.endLine; i++ ) {
+                      c.setLine( i );
+                      c.setColumn( doc()->lineLength( i ) );
+                      doc()->insertText( c, added );
+                  }
+                  break;
+              default:
+                  error("not supported");
+              }
+          }
 
-        m_blockInsert = None;
+          m_blockInsert = None;
+      }
+      else
+      {
+          const QString added = doc()->text(Range(m_viInputModeManager->getMarkPosition('^'), m_view->cursorPosition()));
+
+          if (m_count > 1)
+          {
+              for (unsigned int i = 0; i < m_count - 1; i++)
+              {
+                  if (m_countedRepeatsBeginOnNewLine)
+                  {
+                      doc()->newLine(m_view);
+                  }
+                  doc()->insertText( m_view->cursorPosition(), added );
+              }
+          }
+          m_countedRepeatsBeginOnNewLine = false;
+      }
     }
     startNormalMode();
 }
